#ifndef _CmsToRaveObjects_H_
#define _CmsToRaveObjects_H_

#include <rave/Track.h>
#include <rave/Vertex.h>
#include <rave/Vector3D.h>
#include <rave/Vector6D.h>
#include <rave/Covariance3D.h>
#include <rave/Covariance6D.h>
#include "RecoVertex/VertexPrimitives/interface/TransientVertex.h"
#include "TrackingTools/TransientTrack/interface/TransientTrack.h"
#include "TrackingTools/TrajectoryState/interface/TrajectoryStateOnSurface.h"

#ifdef WITH_KINEMATICS

#include <rave/Vector7D.h>
#include <rave/Covariance7D.h>

#include "RecoVertex/KinematicFitPrimitives/interface/KinematicParameters.h"
#include "RecoVertex/KinematicFitPrimitives/interface/KinematicParametersError.h"

#endif

class GlobalTrajectoryParameters;
class CartesianTrajectoryError;

class CmsToRaveObjects 
{
  public:
    rave::Track convert ( const TrajectoryStateOnSurface &, double chi2=0., double ndf=0., 
                          void * id=0, const std::string & tag="" ) const;
    rave::Track convert ( const FreeTrajectoryState & ) const;
    rave::Track convert ( const reco::TransientTrack & t, int id=0 ) const;
    rave::Vector3D convert ( const GlobalPoint & ) const;
    rave::Vector3D convert ( const GlobalVector & ) const;
    rave::Vector6D convert ( const GlobalTrajectoryParameters & ) const;
    rave::Covariance3D convert ( const GlobalError & ) const;
    rave::Covariance6D convert ( const CartesianTrajectoryError & ) const;
    rave::Vertex convert( const TransientVertex& tv, const std::vector < rave::Track > & ) const;

#ifdef WITH_KINEMATICS
    rave::Vector7D convert( const ::KinematicParameters & ) const;
    rave::Covariance7D convert( const ::KinematicParametersError & ) const;
#endif
};

#endif
