#include "RaveBase/RaveEngine/interface/RaveFlavorTagger.h"
#ifdef WITH_FLAVORTAGGING
#include "RaveBase/Converters/interface/AlgorithmToRaveConverter.h"
#include "RecoBTag/CombinedSV/interface/CombinedSVAlgorithm.h"
// #include "RecoVertex/MultiVertexFit/interface/MultiVertexReconstructor.h"
// #include "RecoVertex/MultiVertexFit/interface/MultiVertexBSeeder.h"
#include "RaveTools/Converters/interface/MagneticFieldSingleton.h"
#include "RecoBTag/CombinedSV/interface/CombinedSVCalibrationCategory.h"
#include "RecoBTag/CombinedSV/interface/LikelihoodRatio.h"
#include "RecoBTag/CombinedSV/interface/CombinedSVTaggingVariables.h"
#include "RecoBTag/CombinedSV/interface/NCategoriesDiscriminatorComputer.h"
#include "DataFormats/BTauReco/interface/TaggingVariable.h"
#include "RecoBTag/CombinedSVLearning/interface/NCategoriesTeacher.h"
#include "RecoBTag/CombinedSV/interface/FilterFactory.h"
#include "RaveBase/Converters/interface/RaveAlgorithmConverter.h"
#include "RecoBTag/CombinedSV/interface/CombinedSVTaggingVariables.h"
#if 0
#include "RecoBTau/JetTagComputer/interface/GenericMVAComputer.h"
#include "PhysicsTools/MVAComputer/interface/MVAComputer.h"
#endif

#include <ctime>
#endif
#include "FWCore/MessageLogger/interface/MessageLogger.h"

using namespace std;

#ifdef WITH_FLAVORTAGGING

namespace {

  struct ThreeCalibrations {
    rave::Calibration reco;
    rave::Calibration pseudo;
    rave::Calibration no;
  };

  ThreeCalibrations splitCalibration ( const rave::Calibration & c )
  {
    ThreeCalibrations ret;
    using namespace reco;
    for ( vector< rave::CalibrationHistogram >::const_iterator i=c.begin(); i!=c.end() ; ++i )
    {
       switch ( i->vertexType() )
       {
         case btag::Vertices::RecoVertex:
           ret.reco.add ( *i ); break;
         case btag::Vertices::PseudoVertex:
           ret.pseudo.add ( *i ); break;
         case btag::Vertices::NoVertex:
           ret.no.add ( *i ); break;
         case btag::Vertices::UndefVertex:
         default:
           edm::LogError("RaveFlavorTagger") << "Iiee, unknown vertex cat!";
       }
    }

    /*
    vector< rave::CalibrationRun > runs=c.runs();

    for ( vector< rave::CalibrationRun >::const_iterator i=runs.begin(); i!=runs.end() ; ++i )
    {
      ret.reco.add ( *i );
      ret.pseudo.add ( *i );
      ret.no.add ( *i );
    }*/

    return ret;
  }

  vector < reco::btau::TaggingVariableName > noVars()
  {
    vector < reco::btau::TaggingVariableName > ret;
    // NoVertex category has TrackIP2DSignificance only.
    ret.push_back ( reco::btau::vertexCategory );
    ret.push_back ( reco::btau::trackSip2d );
    return ret;
  }

  vector < reco::btau::TaggingVariableName > pseudoVars()
  {
    vector < reco::btau::TaggingVariableName > ret=noVars();
    // PseudoVertex category = NoVertex Variables plus the ones below
    ret.push_back ( reco::btau::vertexMass );
    ret.push_back ( reco::btau::vertexMultiplicity );
    ret.push_back ( reco::btau::secondaryVtxEnergyRatio );
    ret.push_back ( reco::btau::trackSip2dAbCharm );
    ret.push_back ( reco::btau::trackEtaRel );
    // ret.push_back ( reco::btau::secondaryVtxWeightedEnergyRatio );
    return ret;
  }

  vector < reco::btau::TaggingVariableName > recoVars()
  {
    vector < reco::btau::TaggingVariableName > ret=pseudoVars();
    // RecoVertex category = PseudoVertex Variables plus flightdistance.
    ret.push_back ( reco::btau::flightDistance2DSignificance );
    // ret.push_back ( reco::btau::jetNVertices );
    return ret;
  }

#if 0
  void addCalibVariables ( PhysicsTools::Calibration::MVAComputer * calib,
                           const vector < reco::btau::TaggingVariableName > & vars )
  {
    PhysicsTools::Calibration::Variable v;
    for ( vector< reco::btau::TaggingVariableName >::const_iterator i=vars.begin(); 
          i!=vars.end() ; ++i )
    {
      v.name=reco::TaggingVariableTokens[*i];
      calib->inputSet.push_back ( v );
      // cout << "[addCalibVariables] adding " << v.name << endl;
    }

  }
#endif
}
#endif

rave::FlavorTagger::FlavorTagger( const rave::Calibration & c, const RaveReconstructor * r )
  #ifdef WITH_FLAVORTAGGING
  : theAlgo(0), theTeacher(0)
  #endif
{
  #ifdef WITH_FLAVORTAGGING
  theMetaInfo = c.runs();

#if 1
  ThreeCalibrations t = splitCalibration ( c );
  RaveAlgorithmConverter converter;
  LikelihoodRatio * recovertex = new LikelihoodRatio ( new BTagAlgorithmCalibration ( converter.convert ( t.reco ) ), reco::btag::Vertices::RecoVertex, 0.25 /* charm prior */ );
  LikelihoodRatio * pseudovertex = new LikelihoodRatio ( new BTagAlgorithmCalibration ( converter.convert ( t.pseudo ) ), reco::btag::Vertices::PseudoVertex, 0.25 /* charm prior */ );
  LikelihoodRatio * novertex = new LikelihoodRatio ( new BTagAlgorithmCalibration ( converter.convert( t.no ) ), reco::btag::Vertices::NoVertex, 0.25 /* charm prior */ );
  NCategoriesDiscriminatorComputer * ncat =
    new NCategoriesDiscriminatorComputer ( recovertex, pseudovertex, novertex );
  theTeacher = new NCategoriesTeacher (
      const_cast < BTagAlgorithmCalibration * > ( &(recovertex->calibration()) ),
      const_cast < BTagAlgorithmCalibration * > ( &(pseudovertex->calibration()) ),
      const_cast < BTagAlgorithmCalibration * > ( &(novertex->calibration())) );
  FilterFactory ff; // use the default filters
  
  // CombinedSVTaggingVariables vars ( sets.recoVars(), sets.pseudoVars(), sets.noVars() );
  CombinedSVTaggingVariables vars ( CombinedSVTaggingVariables::FullSet );
  theAlgo = new combsv::CombinedSVAlgorithm ( MagneticFieldSingleton::Instance(), *r, ncat, vars, ff );
#endif

#if 0
  CombinedSVTaggingVariables vars ( recoVars(), pseudoVars(), noVars() );
  // cout << "[RaveFlavorTagger] debug creating calib" << endl;
  PhysicsTools::Calibration::MVAComputer * calib = 
    new PhysicsTools::Calibration::MVAComputer(); // FIXME who deletes this?
  // cout << "[RaveFlavorTagger] debug instantiate genric mva computer" << endl;
  PhysicsTools::Calibration::ProcTMVA p;
  p.method="ProcTMVA-RaveFlavorTagger";
  calib->addProcessor ( &p );
  calib->output=1;
  addCalibVariables ( calib, recoVars() );

  // cout << "[RaveFlavorTagger] now!" << endl;
  GenericMVAComputer * ncat = 0;
  try {
    ncat = new GenericMVAComputer ( calib );
  } catch ( cms::Exception & e ) {
      cout << "[RaveFlavorTagger] caught: " << e.what() << endl;
    }
  cout << "[RaveFlavorTagger] calib computer has " << calib->getProcessors().size()
       << " processors!" << endl;

  FilterFactory ff; // use the default filters
  theAlgo = new combsv::CombinedSVAlgorithm ( MagneticFieldWrapper::Instance(), *r, ncat, vars, ff );
  theTeacher = 0;
#endif

  #endif
}

rave::FlavorTagger::~FlavorTagger()
{
  #ifdef WITH_FLAVORTAGGING
  delete theAlgo;
  if (theTeacher) delete theTeacher;
  #endif
}

rave::FlavorTagger::FlavorTagger ( const rave::FlavorTagger & o )
  /*
  #ifdef WITH_FLAVORTAGGING
    : theAlgo ( new combsv::CombinedSVAlgorithm ( MultiVertexReconstructor ( MultiVertexBSeeder() ), o.theCalib ) )
  #endif
  */
{
  edm::LogError ("rave::FlavorTagger") << "Somebody tries to copy!";
}

double rave::FlavorTagger::tag (
    const std::vector < reco::TransientTrack > & trks,
    const TransientVertex & v,
    const reco::Particle & p ) const
{
  #ifdef WITH_FLAVORTAGGING
  return theAlgo->tag ( v, p, trks, 1 ).discriminator();
  #else
  cout << "[rave::FlavorTagger] Error: btagging code not available!" << endl;
  return -1.0;
  #endif
}

void rave::FlavorTagger::teach (
    const std::vector < reco::TransientTrack > & trks,
    const TransientVertex & v,
    const reco::Particle & p, char tag ) const
{
  #ifdef WITH_FLAVORTAGGING
  reco::Vertex rv ( v );
  // edm::LogInfo ("rave::FlavorTagger") << "Trying to teach" << endl;
  reco::CombinedSVTagInfo cbtag = theAlgo->tag ( rv, p, trks, 1 /*JTARef*/ );
  if ( cbtag.discriminator() > -.5 )
  {
    theTeacher->teach ( cbtag.variables(), tag );
  } else {
    edm::LogInfo ( "rave::FlavorTagger" ) << "nothing to teach.";
  }
  #else
  cout << "[rave::FlavorTagger] Error: btagging code not available!" << endl;
  #endif
}

rave::Calibration rave::FlavorTagger::getCalibration() const
{
  #ifdef WITH_FLAVORTAGGING
  AlgorithmToRaveConverter rac;
  rave::Calibration ret = rac.convert ( theTeacher->recoCalibration(),
                       theTeacher->pseudoCalibration(),
                       theTeacher->noCalibration() );

  for ( vector< rave::CalibrationRun >::const_iterator i=theMetaInfo.begin();
        i!=theMetaInfo.end() ; ++i )
  {
    ret.add ( *i );
  }
  return ret;
  #else
  cout << "[rave::FlavorTagger] Error: btagging code not available!" << endl;
  return rave::Calibration();
  #endif
}

bool rave::FlavorTagger::hasFlavorTagging()
{
  #ifdef WITH_FLAVORTAGGING
  return true;
  #else
  return false;
  #endif
}

rave::FlavorTagger * rave::FlavorTagger::clone() const
{
  return new FlavorTagger ( * this );
}
