#include "RaveBase/RaveEngine/interface/RaveStrategyBuilder.h"
#include "RecoVertex/VertexPrimitives/interface/VertexReconstructor.h"
#include "RecoVertex/ConfigurableVertexReco/interface/ConfigurableVertexReconstructor.h"
#include "FWCore/ParameterSet/interface/ParameterSet.h"
#include <iostream>
#include <map>

using namespace std;

namespace {
  string::size_type find_unbracketed ( const string & src, const string & sub )
  {
    // find next "sub" (usually: -) that is not within () brackets
    string::size_type open = src.find("(");
    string::size_type close = src.rfind(")");
    string::size_type pos = src.find ( sub );

    // cout << "[find_unbracketed] src=" << src << " sub=" << sub << endl;
    
    if ( open == string::npos ) open=src.size();
    if ( close == string::npos ) close=0;

    // cout << "   open=" << open << " close=" << close << " pos=" << pos << endl;

    // if ( pos == string::npos ) return pos;
    if ( pos > open && pos < close ) pos=string::npos;

    // cout << "   returning " << (signed) pos << endl;
    return pos;
  }

  void print ( const vector < string > & tokens )
  {
    cout << "[RaveStrategyBuilder] tokens: ";
    for ( vector< string >::const_iterator 
          i=tokens.begin(); i!=tokens.end() ; ++i )
    {
      cout << *i << ", ";
    }
    cout << "." << endl;
  }
  
  /*
  void print ( map < string, harvest::MultiType > & data )
  {
    for ( map < string, harvest::MultiType >::iterator i=data.begin(); i!=data.end() ; ++i )                             
    {                                                                                                           
      cout << "  +- " << i->first << ": " << i->second.asString() << endl;                                      
    }                                                                                                           
  } */          

  vector < string > getToken ( string s )
  {
    vector < string > ret;
    // splits Adaptive-cut:3.0 into "Adaptive", "cut:3.0"
    // code is the same as in MultiVertexHistogrammer
    string::size_type pos = find_unbracketed ( s, "-" );
    while ( pos != string::npos )
    {
      string sub = s.substr ( 0, pos );
      s=s.substr ( pos+1, s.size() );
      ret.push_back ( sub );
      // cout << "  -- pushing " << sub << endl;
      pos=find_unbracketed ( s, "-" ); 
    };
    // cout << "  -- pushing " << s << endl;
    ret.push_back ( s );
    // print ( ret );
    return ret;
  }

  edm::ParameterSet getDescription( string s )
  {
    // transforms Adaptive-cut:3.0-linpt:Fsmw into
    // "fitter":"Adaptive", "cut":3.0, "linpt":"Fsmw"
    // code is the same as in MultiVertexHistogrammer

    edm::ParameterSet ret;
    // FIXME produce nested ParameterSets.

    vector < string > d = getToken ( s );
    for ( vector< string >::const_iterator i=d.begin(); i!=d.end() ; ++i )
    {
      if ( (*i)=="" ) continue;
      string::size_type pos = i->find (":");
      if ( pos == string::npos )
      {
        ret.addParameter<string>("finder",(*i));
      } else {
        string name = (*i).substr ( 0, pos );
        string value = (*i).substr ( pos+1, (*i).size() );
        // cout << "[RaveStrategyBuilder] name,value=" << name << "," << value << endl;
        if ( value.find("(") != string::npos )
        {
          string::size_type op = value.find("(");
          string::size_type cl = value.rfind(")");
          string sub = value.substr(op+1,cl-1);
          // cout << "[RaveStrategyBuilder] sub=" << sub << endl;
          ret.addParameter<edm::ParameterSet>(name,getDescription(sub));
        } else {
          ret.addParameter<string>(name,value); 
        }
      };
    };
    return ret;
  }
}

RaveStrategyBuilder::RaveStrategyBuilder ( const string & description, int verbosity ) :
    theRector(0), theVerbosity ( verbosity )
{
  // now split description
  edm::ParameterSet set = getDescription ( description );
  // cout << "[RaveStrategyBuilder] got: " << set << endl;

  if ( theVerbosity )
  {
    cout << "[RaveStrategyBuilder] got: " << set << endl;
  }
  // now build the reconstructor.
  // call the ConfigurableVertexReconstructor, which calls the
  // ConfigurableVertexFitter, ....
  theRector = new ConfigurableVertexReconstructor ( set );
}

RaveStrategyBuilder::RaveStrategyBuilder ( const RaveStrategyBuilder & o ) :
  theRector ( o.theRector->clone() )
{}

RaveStrategyBuilder::~RaveStrategyBuilder()
{
  // the builder does not own
  // if ( theRector ) delete theRector;
}

const VertexReconstructor * RaveStrategyBuilder::reconstructor() const
{
  return theRector;
}
