#include "RaveBase/RaveInterface/rave/Covariance6D.h"

namespace rave
{

Covariance6D::Covariance6D ( double dxx, double dxy, double dxz,
                             double dyy, double dyz, double dzz,
                             double dxpx, double dxpy, double dxpz,
                             double dypx, double dypy, double dypz,
                             double dzpx, double dzpy, double dzpz,
                             double dpxpx, double dpxpy, double dpxpz,
                             double dpypy, double dpypz, double dpzpz ) :
    thedxx ( dxx ), thedxy ( dxy ), thedxz ( dxz ), thedyy ( dyy ), thedyz ( dyz ), thedzz ( dzz ),
    thedxpx ( dxpx ), thedxpy ( dxpy ), thedxpz ( dxpz ), thedypx ( dypx ), thedypy ( dypy ), thedypz ( dypz ),
    thedzpx ( dzpx ), thedzpy ( dzpy ), thedzpz ( dzpz ),
    thedpxpx ( dpxpx ), thedpxpy ( dpxpy ), thedpxpz ( dpxpz ),
    thedpypy ( dpypy ), thedpypz ( dpypz ), thedpzpz ( dpzpz )
{}

Covariance6D::Covariance6D() :
    thedxx ( 0. ), thedxy ( 0. ), thedxz ( 0. ), thedyy ( 0. ), thedyz ( 0. ), thedzz ( 0. ),
    thedxpx ( 0. ), thedxpy ( 0. ), thedxpz ( 0. ), thedypx ( 0. ), thedypy ( 0. ), thedypz ( 0. ),
    thedzpx ( 0. ), thedzpy ( 0. ), thedzpz ( 0. ),
    thedpxpx ( 0. ), thedpxpy ( 0. ), thedpxpz ( 0. ),
    thedpypy ( 0. ), thedpypz ( 0. ), thedpzpz ( 0. )
{}


double Covariance6D::dxx() const
{
  return thedxx;
}

double Covariance6D::dxy() const
{
  return thedxy;
}

double Covariance6D::dxz() const
{
  return thedxz;
}

double Covariance6D::dyy() const
{
  return thedyy;
}

double Covariance6D::dyz() const
{
  return thedyz;
}

double Covariance6D::dzz() const
{
  return thedzz;
}

double Covariance6D::dpxpx() const
{
  return thedpxpx;
}

double Covariance6D::dpxpy() const
{
  return thedpxpy;
}

double Covariance6D::dpxpz() const
{
  return thedpxpz;
}

double Covariance6D::dpypy() const
{
  return thedpypy;
}

double Covariance6D::dpypz() const
{
  return thedpypz;
}

double Covariance6D::dpzpz() const
{
  return thedpzpz;
}

double Covariance6D::dxpx() const
{
  return thedxpx;
}

double Covariance6D::dxpy() const
{
  return thedxpy;
}

double Covariance6D::dxpz() const
{
  return thedxpz;
}

double Covariance6D::dypx() const
{
  return thedypx;
}

double Covariance6D::dypy() const
{
  return thedypy;
}

double Covariance6D::dypz() const
{
  return thedypz;
}

double Covariance6D::dzpx() const
{
  return thedzpx;
}

double Covariance6D::dzpy() const
{
  return thedzpy;
}

double Covariance6D::dzpz() const
{
  return thedzpz;
}

bool Covariance6D::equal ( const Covariance6D & other ) const
{
  return ( dxx() == other.dxx() ) &&
         ( dxy() == other.dxy() ) &&
         ( dxz() == other.dxz() ) &&
         ( dyy() == other.dyy() ) &&
         ( dyz() == other.dyz() ) &&
         ( dzz() == other.dzz() ) &&

         ( dxpx() == other.dxpx() ) &&
         ( dxpy() == other.dxpy() ) &&
         ( dxpz() == other.dxpz() ) &&
         ( dypx() == other.dypx() ) &&
         ( dypy() == other.dypy() ) &&
         ( dypz() == other.dypz() ) &&
         ( dzpx() == other.dzpx() ) &&
         ( dzpy() == other.dzpy() ) &&
         ( dzpz() == other.dzpz() ) &&

         ( dpxpx() == other.dpxpx() ) &&
         ( dpxpy() == other.dpxpy() ) &&
         ( dpxpz() == other.dpxpz() ) &&
         ( dpypy() == other.dpypy() ) &&
         ( dpypz() == other.dpypz() ) &&
         ( dpzpz() == other.dpzpz() );
}

} // namespace rave

bool operator== ( const rave::Covariance6D & lhs, const rave::Covariance6D & rhs )
{
  return lhs.equal ( rhs );
}
